/* wptUtil.cpp - Helper functions
 *	Copyright (C) 2005 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

/* code taken from glibc-2.2.1/sysdeps/generic/strsep.c */
char*
strsep (char **stringp, const char *delim)
{
    char *begin, *end;
    begin = *stringp;

    if (begin == NULL)
        return NULL;

    /* A frequent case is when the delimiter string contains only one
       character.  Here we don't need to call the expensive `strpbrk'
       function and instead work using `strchr'.  */
    if (delim[0] == '\0' || delim[1] == '\0') {
        char ch = delim[0];
        if (ch == '\0')
            end = NULL;
        else {
            if (*begin == ch)
                end = begin;                         
            else if (*begin == '\0')
                end = NULL;
            else
                end = strchr (begin + 1, ch);
        }  
    }
    else /* Find the end of the token.  */
        end = strpbrk (begin, delim);
    if (end) {
        /* Terminate the token and set *STRINGP past NUL character.  */
        *end++ = '\0';
        *stringp = end;
    }
    else
        /* No more delimiters; this is the last token.  */
        *stringp = NULL;

    return begin;
}


/* Like strstr but this version is case in-sentensive. */
const char *
stristr (const char *buf, const char *sub)
{
    const char *t, *s ;
    size_t n;
    size_t buflen = strlen (buf);

    for( t=buf, n=buflen, s=sub ; n ; t++, n-- ) {
	if( toupper(*t) == toupper(*s) ) {
	    for( buf=t++, buflen = n--, s++;
		 n && toupper(*t) == toupper(*s); t++, s++, n-- )
		;
	    if( !*s )
		return buf;
	    t = buf; n = buflen; s = sub ;
	}
    }
    return NULL ;
}


/* Check if the email address @email only contain valid characters.
   Return 0 on success. */
int
check_email_address (const char *email)
{
    const char *allowed = "@._-%+;";
    size_t i;

    if (!strchr (email, '@'))
	return -1;
    for (i=0; i < strlen (email); i++) {
	if (isxdigit (email[i]) || isalpha (email[i])
	    || strchr (allowed, email[i]))
	    continue;
	else
	    return -1;
    }
    return 0;
}


/* Return a substring of @str from the position @begin
   to position @end. */
char*
substr (const char *str, unsigned int begin, unsigned int end)
{    
    char *p;
    size_t i, pos;

    if (end > strlen (str) || begin > strlen (str) || (end-begin) < 0)
	return NULL;

    p = new char[end-begin+1];
    if (!p)
	abort ();

    for (i = begin, pos=0; i < end; i++)
	p[pos++] = str[i];
    p[pos] = '\0';
    return p;
}


/* Remove %AB sequences from the input buffer @in
   and store the raw data in @out. */
void
unhexify_buffer (const char *in, char **r_out)
{
    char temp[3], *out;
    size_t len, pos, i=0;

    len = strlen (in);
    out = new char[len+1];
    if (!out)
	abort ();
    memset (out, 0, len+1);
    for (pos = 0; pos < len; pos++) {
	if (in[pos] == '%' && in[pos+1] == '%')
	    out[i++] = '%';
	else if (in[pos] == '%') {
	    temp[0] = in[++pos];
	    temp[1] = in[++pos];
	    temp[2] = 0;
	    out[i++] = (char)strtoul (temp, NULL, 16);
	}
	else
	    out[i++] = in[pos];
    }
    out[i] = 0;
    *r_out = out;
}
